@@ -37,6 +37,10 @@ class GwibberClient(gtk.Window):
     self.ui = gtk.Builder()
     self.ui.set_translation_domain ("gwibber")
 
+    self.connection = microblog.util.getbus("Connection")
+    self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
+    self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
+
     self.model = gwui.Model()
 
     self.service = self.model.daemon
@@ -48,9 +52,8 @@ class GwibberClient(gtk.Window):
 
     self.messages.connect_to_signal("Message", self.on_new_message)
 
-    self.connection = microblog.util.getbus("Connection")
-    self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
-    self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
+    self.shortener = microblog.util.getbus("URLShorten")
+    self.uploader = microblog.util.getbus("Uploader")
 
     self.accounts = microblog.util.getbus("Accounts")
     self.quicklist_messages = {}
@@ -219,11 +222,21 @@ class GwibberClient(gtk.Window):
 
     self.input_splitter.set_focus_child(self.input)
 
+    self.attach_image = gtk.Image()
+    self.attach_image.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON)
+
+    self.button_attach = gtk.Button()
+    self.button_attach.set_image(self.attach_image)
+    self.button_attach.set_tooltip_text(_("Upload image"))
+    self.button_attach.connect("clicked", self.on_button_attach_clicked)
+
     self.button_send = gtk.Button(_("Send"))
+    self.button_send.set_tooltip_text(_("Post to all enabled services"))
     self.button_send.connect("clicked", self.on_button_send_clicked)
 
     self.message_target = gwui.AccountTargetBar(self.model)
     self.message_target.pack_end(self.button_send, False)
+    self.message_target.pack_end(self.button_attach, False)
     self.message_target.connect("canceled", self.on_reply_cancel)
 
     content = gtk.VBox(spacing=5)
@@ -563,6 +576,67 @@ class GwibberClient(gtk.Window):
     self.send_message(text)
     self.input.clear()
 
+  def on_button_attach_clicked(self, w):
+    file_upload = gtk.FileChooserDialog(title=_("Select file to upload"),
+                      action=gtk.FILE_CHOOSER_ACTION_OPEN,
+                      buttons=(gtk.STOCK_CANCEL,
+                          gtk.RESPONSE_CANCEL,
+                          gtk.STOCK_OPEN,
+                          gtk.RESPONSE_OK))
+    file_upload.set_default_response(gtk.RESPONSE_OK)
+    file_upload.connect("response", self.on_button_attach_file_selected)
+    file_upload.show()
+
+  def on_button_attach_file_selected(self, file_upload, response):
+    attachment = None
+    #import epdb;epdb.st()
+    if response == gtk.RESPONSE_CANCEL:
+      file_upload.destroy()
+      return
+
+    if response == gtk.RESPONSE_OK:
+      attachment = file_upload.get_filename()
+    file_upload.destroy()
+
+    if attachment:
+      buf = self.input.textview.get_buffer()
+      # set a mark at iter, so that the callback knows where to insert
+      iter = buf.get_end_iter()
+      mark_start = buf.create_mark(None, iter, True)
+      # insert a placeholder character
+      buf.insert(iter, u"\u2328")
+      # can't just get_insert() because that gets the *named* mark "insert"
+      # and we want an anonymous mark so it won't get changed later
+      iter_end = buf.get_iter_at_mark(buf.get_insert())
+      mark_end = buf.create_mark(None, iter_end, True)
+      gobject.idle_add(self.upload_attachment, attachment, iter, mark_start, mark_end)
+
+  def upload_attachment(self, attachment, iter, mark_start, mark_end):
+    buf = self.input.textview.get_buffer()
+
+    def add_shortened(shortened_url):
+      "Internal add-shortened-url-to-buffer function: a closure"
+      iter_start = buf.get_iter_at_mark(mark_start)
+      iter_end = buf.get_iter_at_mark(mark_end)
+      buf.delete(iter_start, iter_end)
+      buf.insert(iter_start, ("%s " % shortened_url))
+      #buf.insert_at_cursor(" ")
+    def error_shortened(dbus_exc):
+      "Internal shortening-url-died function: a closure"
+      iter = buf.get_iter_at_mark(mark)
+      buf.insert(iter, text) # shortening failed
+
+    text = self.uploader.Upload(attachment)
+
+    if self.model.settings["shorten_urls"]:
+      if text and text.startswith("http") and not " " in text \
+          and len(text) > 30:
+        self.shortener.Shorten(text,
+          reply_handler=add_shortened,
+          error_handler=error_shortened)
+    else:
+      add_shortened(text)
+
   def on_button_send_clicked(self, w):
     self.send_message(self.input.get_text())
     self.input.clear()
