@@ -1071,7 +1071,7 @@ class InputTextView(gtk.TextView):
             iter_start = buf.get_iter_at_mark(mark_start)
             iter_end = buf.get_iter_at_mark(mark_end)
             buf.delete(iter_start, iter_end)
-            buf.insert(iter_start, shortened_url)
+            buf.insert(iter_start, ("%s " % shortened_url))
         def error_shortened(dbus_exc):
             "Internal shortening-url-died function: a closure"
             iter = buf.get_iter_at_mark(mark)
