@@ -117,7 +117,14 @@ class AccountWidget(gtk.VBox):
     url = web.get_main_frame().get_uri()
     if title.get_title() == "Success":
       url = web.get_main_frame().get_uri()
-      self.account["access_token"] = str(urlparse.parse_qs(url.split("#", 1)[1])["access_token"][0])
+      try:
+        self.account["access_token"] = str(urlparse.parse_qs(url.split("#", 1)[1])["access_token"][0])
+      except:
+        web.hide()
+        self.ui.get_object("vbox1").show()
+        self.ui.get_object("vbox_advanced").show()
+        return
+
       data = json.loads(urllib.urlopen("https://graph.facebook.com/me?access_token=" + self.account["access_token"]).read())
       if isinstance(data, dict):
         if data.has_key("id") and data.has_key("name"):
@@ -137,9 +144,6 @@ class AccountWidget(gtk.VBox):
           else:
             print "Failed"
 
-     
-        
-
       self.ui.get_object("hbox_facebook_auth").hide()
       self.ui.get_object("fb_auth_done_label").set_label(_("%s has been authorized by Facebook") % str(self.account["username"]))
       self.ui.get_object("hbox_facebook_auth_done").show()
